package com.cj.jiaqingjiayi.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cj.jiaqingjiayi.common.BaseResponse;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.common.ResultUtils;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.domain.Business;
import com.cj.jiaqingjiayi.model.domain.BusinessAuth;
import com.cj.jiaqingjiayi.model.domain.User;
import com.cj.jiaqingjiayi.model.request.business.BusinessAddRequest;
import com.cj.jiaqingjiayi.model.request.business.BusinessQueryRequest;
import com.cj.jiaqingjiayi.model.request.business.BusinessUpdateRequest;
import com.cj.jiaqingjiayi.model.vo.BusinessAdminVO;
import com.cj.jiaqingjiayi.model.vo.BusinessVO;
import com.cj.jiaqingjiayi.service.BusinessAuthService;
import com.cj.jiaqingjiayi.service.BusinessService;
import com.cj.jiaqingjiayi.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import java.util.List;

import static com.cj.jiaqingjiayi.contant.UserConstant.USER_SALT;

/**
 * 商家接口
 */
@Slf4j
@RestController
@RequestMapping("/business")
public class BusinessController {

    @Resource
    private UserService userService;

    @Resource
    private BusinessService businessService;

    @Resource
    private BusinessAuthService businessAuthService;

    @PostMapping("/add")
    public BaseResponse<Long> addBusiness(@RequestBody BusinessAddRequest businessAddRequest) {
        ThrowUtils.throwIf(businessAddRequest == null, ErrorCode.PARAMS_ERROR);

        User user = new User();
        BeanUtils.copyProperties(businessAddRequest, user);
        Business business = new Business();
        BeanUtils.copyProperties(businessAddRequest, business);
        BusinessAuth businessAuth = new BusinessAuth();
        BeanUtils.copyProperties(businessAddRequest, businessAuth);

        //对每个应该校验的业务进行校验
        businessService.validUser(user, true);
        businessService.validBusiness(business, true);
        businessService.validBusinessAuth(businessAuth, true);

        //加密
        String encryptPassword = DigestUtils.md5DigestAsHex((USER_SALT + user.getUserPassword()).getBytes());
        user.setUserPassword(encryptPassword);

        //更改用户权限为商家
        user.setUserRole(2);

        //往user表中补充商家信息
        user.setUsername(business.getBusinessName());
        user.setAvatarUrl(business.getBusinessAvatar());
        user.setPhone(business.getBusinessPhone());

        //插入到表中
        businessService.addBusiness(user, business, businessAuth);

        return ResultUtils.success(business.getId(),"添加商家成功");

    }

    /**
     * 管理员更新商家
     * @param businessUpdateRequest
     *  @return
     */
    @PostMapping("/update")
    public BaseResponse<Boolean> updateBusiness (@RequestBody BusinessUpdateRequest businessUpdateRequest, HttpServletRequest request) {
        //判断是否为管理员
        userService.isAdmin(request);

        if (businessUpdateRequest == null || businessUpdateRequest.getId() == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "传入参数为空");
        }

        Business business = new Business();

        BeanUtils.copyProperties(businessUpdateRequest, business);
        //更新不添加 只需要判断修改符不符合逻辑
        businessService.validBusiness(business, false);

        boolean updateById = businessService.updateById(business);

        ThrowUtils.throwIf(!updateById, ErrorCode.OPERATION_ERROR);

        return ResultUtils.success(true);
    }

    /**
     * 根据id查询
     * @param id
     * @return
     */
    @GetMapping("/getById")
    public BaseResponse<BusinessAdminVO> getBusinessById(long id, HttpServletRequest request) {
        //判断是否为管理员
        userService.isAdmin(request);
        ThrowUtils.throwIf(id < 0, ErrorCode.NULL_ERROR);

        //获取商家视图
        Business business = businessService.getById(id);
        ThrowUtils.throwIf(business == null , ErrorCode.PARAMS_ERROR, "id不符合规范");
        BusinessAdminVO businessAdminVO = businessAuthService.getBusinessAdminVO(business);

        return ResultUtils.success(businessAdminVO);
    }

    /**
     * 查询所有商家
     * @return
     */
    @PostMapping("/list")
    public BaseResponse<List<Business>> listBusinessAll(){
        List<Business> businessList = businessService.list();
        return new BaseResponse<>(0,businessList,"查询成功");
    }


    /**
     * 分页查询
     * @param businessQueryRequest
     * @return
     */
    @PostMapping("/list/page")
    //@AuthCheck(mustRole = 1)
    public BaseResponse<Page<Business>> listBusinessByPage (@RequestBody BusinessQueryRequest businessQueryRequest) {
        Page<Business> businessPage = businessService.page(new Page<>(businessQueryRequest.getCurrent(), businessQueryRequest.getPageSize()),
                businessService.getQueryWrapper(businessQueryRequest));
        return ResultUtils.success(businessPage);

    }

    /**
     * 分页获取商家列表（用户视图）
     */
    @PostMapping("/list/page/vo")
    public BaseResponse<Page<BusinessVO>> listBusinessVOByPage(@RequestBody BusinessQueryRequest businessQueryRequest) {
        long current = businessQueryRequest.getCurrent();
        long size = businessQueryRequest.getPageSize();

        ThrowUtils.throwIf(size > 20, ErrorCode.PARAMS_ERROR);
        QueryWrapper<Business> queryWrapper = businessService.getQueryWrapper(businessQueryRequest);
        // 只显示正常营业商家
        queryWrapper.eq("state", 1);
        Page<Business> businessPage = businessService.page(new Page<>(current, size), queryWrapper);
        Page<BusinessVO> businessVOPage = new Page<>(current, size, businessPage.getTotal());
        List<BusinessVO> businessVOList = businessService.getBusinessVO(businessPage.getRecords());
        businessVOPage.setRecords(businessVOList);
        return ResultUtils.success(businessVOPage);
    }


    @GetMapping("/current")
    public BaseResponse<BusinessVO> getLoginBusiness(HttpServletRequest request){
        //获取登录态
        Business business = businessService.getLoginBusiness(request);
        return ResultUtils.success(businessService.getBusinessVO(business));
    }
    

}
