package com.cj.jiaqingjiayi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.domain.Business;
import com.cj.jiaqingjiayi.model.domain.Cart;
import com.cj.jiaqingjiayi.model.domain.Commodities;
import com.cj.jiaqingjiayi.model.domain.User;
import com.cj.jiaqingjiayi.model.request.cart.CartAddRequest;
import com.cj.jiaqingjiayi.model.request.cart.CartUpdateRequest;
import com.cj.jiaqingjiayi.service.BusinessService;
import com.cj.jiaqingjiayi.service.CartService;
import com.cj.jiaqingjiayi.mapper.CartMapper;
import com.cj.jiaqingjiayi.service.CommoditiesService;
import com.cj.jiaqingjiayi.service.UserService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;

/**
* @author 高木
* @description 针对表【cart(购物车表)】的数据库操作Service实现
* @createDate 2024-09-05 15:15:56
*/
@Service
public class CartServiceImpl extends ServiceImpl<CartMapper, Cart>
    implements CartService{

    @Resource
    private UserService userService;

    @Resource
    private BusinessService businessService;

    @Resource
    private CommoditiesService commoditiesService;

    @Override
    public void validCart(CartAddRequest cartAddRequest) {

        if (cartAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }

        Long businessId = cartAddRequest.getBusinessId();
        Long commoditiesId = cartAddRequest.getCommoditiesId();
        Integer quantity = cartAddRequest.getQuantity();

        if (quantity == null || quantity < 1) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR,"商品数量错误");
        }

        Business business = businessService.getById(businessId);
        if (business == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"不存在该商家");
        }

        Commodities commodities = commoditiesService.getById(commoditiesId);
        if (commodities == null || commodities.getStatus().equals("下架")){
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"不存在该商品");
        }
    }

    @Override
    public Long addCart(CartAddRequest cartAddRequest, HttpServletRequest request) {
        User loginUser = userService.getLogingUser(request);
        Long userId = loginUser.getId();
        Long businessId = cartAddRequest.getBusinessId();
        Long commoditiesId = cartAddRequest.getCommoditiesId();
        //计算商品价格
        Commodities commodities = commoditiesService.getById(commoditiesId);
        BigDecimal price = new BigDecimal(String.valueOf(commodities.getCommoditiesPrice()));
        //保存购物车商品
        Cart cart = new Cart();
        cart.setBusinessId(businessId);
        cart.setUserId(userId);
        cart.setPrice(price);
        cart.setCommoditiesId(commoditiesId);
        boolean save = this.save(cart);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);

        return cart.getId();
    }

    @Override
    public Boolean updateCart(CartUpdateRequest cartUpdateRequest, HttpServletRequest request) {
        User loginUser = userService.getLogingUser(request);
        Long userId = loginUser.getId();
        Long commoditiesId = cartUpdateRequest.getCommoditiesId();
        String selectedOptions = cartUpdateRequest.getSelectedOptions();
        Long id = cartUpdateRequest.getId();
        ThrowUtils.throwIf(id == null,ErrorCode.OPERATION_ERROR,"购物车id不存在");
        //检验该购物车是否为登录用户的
        Cart cartId =this.getById(id);
        Long cartUserId = cartId.getUserId();
        if (userId.equals(cartUserId)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR,"无法操作该购物车");
        }
        Commodities commodities = commoditiesService.getById(commoditiesId);
        BigDecimal price = new BigDecimal(String.valueOf(commodities.getCommoditiesPrice()));
        LambdaQueryWrapper<Cart> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Cart::getId, id);
        Cart cart = this.getOne(wrapper);
        Integer quantity = cartUpdateRequest.getQuantity();
        if (cartUpdateRequest.getIsSelectedOptions()) {
            cart.setQuantity(cart.getQuantity() + quantity);
        } else {
            cart.setQuantity(quantity);
        }
        cart.setPrice(price);
        cart.setSubtotal(null);
        cart.setSelectedOptions(selectedOptions);

        return this.updateById(cart);
    }

    @Override
    public Boolean deleteCart(Long cartId, HttpServletRequest request) {
        User loginUser = userService.getLogingUser(request);
        Long userId = loginUser.getId();
        Cart cart = this.getById(cartId);
        Long cartUserId = cart.getUserId();

        if (!userId.equals(cartUserId)) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "该用户无权删除该购物车");
        }
        boolean remove = this.removeById(cartId);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);

        return true;
    }


}




