package com.cj.jiaqingjiayi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.mapper.UserMapper;
import com.cj.jiaqingjiayi.model.domain.User;
import com.cj.jiaqingjiayi.model.request.user.UserSearchRequest;
import com.cj.jiaqingjiayi.model.request.user.UserUpdatePasswordRequest;
import com.cj.jiaqingjiayi.model.vo.LoginUserVO;
import com.cj.jiaqingjiayi.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import static com.cj.jiaqingjiayi.contant.UserConstant.*;


@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserMapper, User>
        implements UserService {

    @Resource
    private UserMapper userMapper;

    /**
     * 盐值 对密码进行混淆加密
     */
    private static final String SALT = "yctf";

    /**
     * 用户注册
     *
     * @param userAccount   登陆账号
     * @param userPassword  登录密码
     * @param checkPassword 校验密码
     * @return 新id
     */

    @Override
    public long userRegister(String userAccount, String userPassword, String checkPassword) {

        //1.非空
        if (StringUtils.isAllBlank(userAccount, userPassword, checkPassword)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数为空");
        }
        //2.账号长度不小于4位
        if (userAccount.length() < 4) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "用户账号过短");
        }
        //3.密码长度不小于8位
        if (userPassword.length() < 8 || checkPassword.length() < 8) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "用户密码过短");
        }
        //正则表达式看账户包不包含特殊字符
        String validPattern = "[`~!@#$%^&*()+=|{}':;',\\\\[\\\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。，、？]";

        Matcher matcher = Pattern.compile(validPattern).matcher(userAccount);

        if (matcher.find()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "不符合规范");
        }

        //检验密码和校验密码是否相同
        if (!userPassword.equals(checkPassword)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "两次输入密码不同");
        }

        //账户不能重复，进行查询数据库看当中是否存在相同名称用户
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userAccount", userAccount);
        Long count = userMapper.selectCount(queryWrapper);
        if (count > 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在相同用户");
        }

        //对密码加密
        String encryptPassword = DigestUtils.md5DigestAsHex((SALT + userPassword).getBytes());

        User user = new User();
        user.setUserAccount(userAccount);
        user.setUserPassword(encryptPassword);
        boolean saveResult = this.save(user);
        if (!saveResult) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "插入失败！");
        }
        return user.getId();
    }

    /**
     * 登录
     *
     * @param userAccount  登陆账号
     * @param userPassword 登陆密码
     * @param request      返回请求
     * @return 用户信息
     */

    @Override
    public LoginUserVO userLogin(String userAccount, String userPassword, HttpServletRequest request) {
        //非空
        if (StringUtils.isAllBlank(userAccount, userPassword)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数为空");
        }
        //账号长度不小于四位
        if (userAccount.length() < 4) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "用户账号过短");
        }
        if (userPassword.length() < 8) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "用户密码过短");
        }

        //正则表达式看账户包不包含特殊字符
        String validPattern = "[`~!@#$%^&*()+=|{}':;',\\\\[\\\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。，、？]";

        Matcher matcher = Pattern.compile(validPattern).matcher(userAccount);

        if (matcher.find()) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "不符合规范");
        }

        //校验账号密码是否相同
        String encodePassword = DigestUtils.md5DigestAsHex((SALT + userPassword).getBytes(StandardCharsets.UTF_8));
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userAccount", userAccount);
        queryWrapper.eq("userPassword", encodePassword);
        User user = userMapper.selectOne(queryWrapper);
        //用户不存在
        if (user == null) {
            log.info("user login failed, userAccount Cannot match userPassword");
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "用户账号或密码有误！");
        }

        //用户登陆成功
        request.getSession().setAttribute(USER_LOGIN_STATE, user);
        return this.getLoginUserVO(user);
    }

    /**
     * 实现用户注销
     * @param request
     * @return
     */
    @Override
    public int userLogout(HttpServletRequest request) {
        request.getSession().removeAttribute(USER_LOGIN_STATE);
        return 1;
    }

    /**
     * 用户脱敏
     * @param user
     * @return
     */
    @Override
    public User getSafetyUser(User user) {
        User safeUser = new User();
        safeUser.setId(user.getId());
        safeUser.setUsername(user.getUsername());
        safeUser.setUserAccount(user.getUserAccount());
        safeUser.setAvatarUrl(user.getAvatarUrl());
        safeUser.setGender(user.getGender());
        safeUser.setPhone(user.getPhone());
        safeUser.setEmail(user.getEmail());
        safeUser.setUserStatus(user.getUserStatus());
        safeUser.setCreateTime(user.getCreateTime());
        return safeUser;
    }

    /**
     * 获取当前登录用户的信息
     * @param request
     * @return
     */
    @Override
    public User getLogingUser(HttpServletRequest request) {
        Object userObj = request.getSession().getAttribute(USER_LOGIN_STATE);
        User currentUser = (User) userObj;

        if(currentUser == null){
            throw new BusinessException(ErrorCode.NOT_LOGIN, "未登录");
        }

        Long userId = currentUser.getId();
        currentUser = this.getById(userId);
        return currentUser;
    }

    /**
     * 用户修改密码
     * @param updatePasswordRequest
     * @param request
     * @return
     */
    @Override
    public boolean updateUserPassword(UserUpdatePasswordRequest updatePasswordRequest, HttpServletRequest request) {
        if (updatePasswordRequest == null){
            throw new BusinessException(ErrorCode.NULL_ERROR);
        }

        /**
         * 判断用户是否存在
         */
        User logingUser = getLogingUser(request);
        Long userId = logingUser.getId();
        if(userId < 0 || userId == null){
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "用户不存在");
        }
        String newPassword = updatePasswordRequest.getNewPassword();

        /**
         * 确认密码
         */
        if(!newPassword.equals(updatePasswordRequest.getCheckNewPassword())){
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "两次输入密码不相同");
        }

        /**
         * 判断修改后的密码是否与原密码相同
         */
        User user = new User();
        BeanUtils.copyProperties(updatePasswordRequest, user);
        user.setId(logingUser.getId());
        if (newPassword.equals(updatePasswordRequest.getUserPassword())){
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "修改后的密码与原密码不能相同");
        }


        //使用MD5加密新密码
        String mdedPassword = DigestUtils.md5DigestAsHex((USER_SALT + updatePasswordRequest.getNewPassword()).getBytes());
        String odlPassword = DigestUtils.md5DigestAsHex((USER_SALT + updatePasswordRequest.getUserPassword()).getBytes());
        if(!logingUser.getUserPassword().equals(odlPassword)){
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "旧密码错误");
        }
        user.setUserPassword(mdedPassword);
        boolean result = updateById(user);
        if(!result){
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "修改失败");
        }
        return true;
    }

    @Override
    /**
     * 分页查询
     *
     * @param searchRequest
     * @return
     */
    public QueryWrapper<User> getQueryWrapper(UserSearchRequest searchRequest) {

        if (searchRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }
        String username = searchRequest.getUsername();
        String userAccount = searchRequest.getUserAccount();
        Integer gender = searchRequest.getGender();
        String phone = searchRequest.getPhone();
        String email = searchRequest.getEmail();
        Integer userStatus = searchRequest.getUserStatus();
        Integer userRole = searchRequest.getUserRole();
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        Date updateTime = searchRequest.getUpdateTime();
        Date createTime = searchRequest.getCreateTime();
        // username
        if (StringUtils.isNotBlank(username)) {
            queryWrapper.like("username", username);
        }
        // userAccount
        if (StringUtils.isNotBlank(userAccount)) {
            queryWrapper.like("userAccount", userAccount);
        }
        // gender
        if (StringUtils.isNotBlank(username)) {
            queryWrapper.eq("gender", gender);
        }
        // phone
        if (StringUtils.isNotBlank(phone)) {
            queryWrapper.like("phone", phone);
        }
        // email
        if (StringUtils.isNotBlank(email)) {
            queryWrapper.like("email", email);
        }
        // userStatus
        if (userStatus != null) {
            queryWrapper.eq("userStatus", userStatus);
        }
        if (userRole != null) {
            queryWrapper.eq("userRole", userRole);
        }
        if (updateTime != null) {
            queryWrapper.like("updateTime", updateTime);
        }
        if (createTime != null) {
            queryWrapper.like("createTime", createTime);
        }
        return queryWrapper;
    }

    @Override
    public LoginUserVO getLoginUserVO(User user) {
        if (user == null) {
            return null;
        }
        LoginUserVO loginUserVO = new LoginUserVO();
        BeanUtils.copyProperties(user, loginUserVO);
        return loginUserVO;
    }

    /**
     * 判断是否为管理员
     * @return
     */
    @Override
    public void isAdmin(HttpServletRequest request){
        Object userObj = request.getSession().getAttribute(USER_LOGIN_STATE);
        User user = (User) userObj;
        ThrowUtils.throwIf(!(user != null && user.getUserRole() == ADMIN_ROLE) ,ErrorCode.NO_AUTH, "无管理员权限");
    }

}